@@ -13,22 +13,15 @@
 #include <QFileInfo>
 #include <QGuiApplication>
 
-#include <KDirWatch>
 #include <KIconLoader>
 #include <KSharedConfig>
 #include <KWindowEffects>
 #include <KWindowSystem>
 #include <KX11Extras>
 
-#include "libplasma-theme-global.h"
-
 namespace Plasma
 {
 const char ThemePrivate::defaultTheme[] = "default";
-const char ThemePrivate::themeRcFile[] = "plasmarc";
-// the system colors theme is used to cache unthemed svgs with colorization needs
-// these svgs do not follow the theme's colors, but rather the system colors
-const char ThemePrivate::systemColorsTheme[] = "internal-system-colors";
 
 ContrastEffectWatcher *ThemePrivate::s_backgroundContrastEffectWatcher = nullptr;
 
@@ -87,8 +80,6 @@ ThemePrivate::ThemePrivate(QObject *parent)
     , defaultWallpaperSuffix(QStringLiteral(DEFAULT_WALLPAPER_SUFFIX))
     , defaultWallpaperWidth(DEFAULT_WALLPAPER_WIDTH)
     , defaultWallpaperHeight(DEFAULT_WALLPAPER_HEIGHT)
-    , cacheSize(0)
-    , cachesToDiscard(NoCache)
     , compositingActive(true)
     , backgroundContrastActive(KWindowEffects::isEffectAvailable(KWindowEffects::BackgroundContrast))
     , isDefault(true)
@@ -109,7 +100,6 @@ ThemePrivate::ThemePrivate(QObject *parent)
         compositingActive = KX11Extras::self()->compositingActive();
     }
 
-    ThemeConfig config;
     kSvgImageSet = std::unique_ptr<KSvg::ImageSet>(new KSvg::ImageSet);
     kSvgImageSet->setBasePath(QStringLiteral(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/"));
 
@@ -120,7 +110,8 @@ ThemePrivate::ThemePrivate(QObject *parent)
     selectorsUpdateTimer->setSingleShot(true);
     selectorsUpdateTimer->setInterval(600);
     QObject::connect(selectorsUpdateTimer, &QTimer::timeout, this, [this]() {
-        updateKSvgSelectors(PixmapCache | SvgElementsCache);
+        updateKSvgSelectors();
+        scheduleThemeChangeNotification();
     });
 
     updateNotificationTimer = new QTimer(this);
@@ -138,22 +129,20 @@ ThemePrivate::ThemePrivate(QObject *parent)
     }
     QCoreApplication::instance()->installEventFilter(this);
 
-    const QString configFile = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1Char('/') + QLatin1String(themeRcFile);
-    KDirWatch::self()->addFile(configFile);
+    plasmaRcWatcher = KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("plasmarc")));
 
-    // Catch both, direct changes to the config file ...
-    connect(KDirWatch::self(), &KDirWatch::dirty, this, &ThemePrivate::settingsFileChanged);
-    // ... but also remove/recreate cycles, like KConfig does it
-    connect(KDirWatch::self(), &KDirWatch::created, this, &ThemePrivate::settingsFileChanged);
+    connect(plasmaRcWatcher.get(), &KConfigWatcher::configChanged, this, [this] {
+        settingsChanged(true);
+    });
 
     QObject::connect(KIconLoader::global(), &KIconLoader::iconChanged, this, [this]() {
-        scheduleThemeChangeNotification(PixmapCache | SvgElementsCache);
+        scheduleThemeChangeNotification();
     });
 
     if (KWindowSystem::isPlatformX11()) {
         connect(KX11Extras::self(), &KX11Extras::compositingChanged, selectorsUpdateTimer, qOverload<>(&QTimer::start));
     }
-    updateKSvgSelectors(NoCache);
+    updateKSvgSelectors();
 }
 
 ThemePrivate::~ThemePrivate()
@@ -175,7 +164,7 @@ KConfigGroup &ThemePrivate::config()
                 groupName.append(QLatin1Char('-')).append(app);
             }
         }
-        cfg = KConfigGroup(KSharedConfig::openConfig(QFile::decodeName(themeRcFile)), groupName);
+        cfg = plasmaRcWatcher->config()->group(groupName);
     }
 
     return cfg;
@@ -187,15 +176,8 @@ QString ThemePrivate::imagePath(const QString &theme, const QString &type, const
     return QStandardPaths::locate(QStandardPaths::GenericDataLocation, subdir);
 }
 
-QString ThemePrivate::findInTheme(const QString &image, const QString &theme, bool cache)
+QString ThemePrivate::findInTheme(const QString &image, const QString &theme)
 {
-    if (cache) {
-        auto it = discoveries.constFind(image);
-        if (it != discoveries.constEnd()) {
-            return it.value();
-        }
-    }
-
     QString type = QStringLiteral("/");
     if (!compositingActive) {
         type = QStringLiteral("/opaque/");
@@ -210,14 +192,10 @@ QString ThemePrivate::findInTheme(const QString &image, const QString &theme, bo
         search = imagePath(theme, QStringLiteral("/"), image);
     }
 
-    if (cache && !search.isEmpty()) {
-        discoveries.insert(image, search);
-    }
-
     return search;
 }
 
-void ThemePrivate::updateKSvgSelectors(CacheTypes notify)
+void ThemePrivate::updateKSvgSelectors()
 {
 #if HAVE_X11
     if (KWindowSystem::isPlatformX11()) {
@@ -239,23 +217,11 @@ void ThemePrivate::updateKSvgSelectors(CacheTypes notify)
     } else {
         kSvgImageSet->setSelectors({QStringLiteral("opaque")});
     }
-
-    if (notify != NoCache) {
-        scheduleThemeChangeNotification(notify);
-    }
-}
-
-void ThemePrivate::discardCache(CacheTypes caches)
-{
-    if (caches & SvgElementsCache) {
-        discoveries.clear();
-    }
 }
 
 void ThemePrivate::colorsChanged()
 {
     // in the case the theme follows the desktop settings, refetch the colorschemes
-    // and discard the svg pixmap cache
     if (colors != nullptr) {
         colors->reparseConfiguration();
     } else {
@@ -269,38 +235,20 @@ void ThemePrivate::colorsChanged()
     headerColorScheme = KColorScheme(QPalette::Active, KColorScheme::Header, colors);
     tooltipColorScheme = KColorScheme(QPalette::Active, KColorScheme::Tooltip, colors);
     palette = KColorScheme::createApplicationPalette(colors);
-    scheduleThemeChangeNotification(PixmapCache | SvgElementsCache);
+    scheduleThemeChangeNotification();
     Q_EMIT applicationPaletteChange();
 }
 
-void ThemePrivate::scheduleThemeChangeNotification(CacheTypes caches)
+void ThemePrivate::scheduleThemeChangeNotification()
 {
-    cachesToDiscard |= caches;
     updateNotificationTimer->start();
 }
 
 void ThemePrivate::notifyOfChanged()
 {
-    // qCDebug(LOG_PLASMA) << cachesToDiscard;
-    discardCache(cachesToDiscard);
-    cachesToDiscard = NoCache;
     Q_EMIT themeChanged();
 }
 
-void ThemePrivate::settingsFileChanged(const QString &file)
-{
-    qCDebug(LOG_PLASMA) << "settingsFile: " << file;
-    if (file == themeMetadataPath) {
-        const KPluginMetaData data = metaDataForTheme(themeName);
-        if (!data.isValid() || themeVersion != data.version()) {
-            scheduleThemeChangeNotification(SvgElementsCache);
-        }
-    } else if (file.endsWith(QLatin1String(themeRcFile))) {
-        config().config()->reparseConfiguration();
-        settingsChanged(true);
-    }
-}
-
 void ThemePrivate::settingsChanged(bool emitChanges)
 {
     if (fixedName) {
@@ -467,19 +415,14 @@ void ThemePrivate::setThemeName(const QString &tempThemeName, bool writeSettings
         }
     }
 
-    // we have one special theme: essentially a dummy theme used to cache things with
-    // the system colors.
-    bool realTheme = theme != QLatin1String(systemColorsTheme);
-    if (realTheme) {
-        KPluginMetaData data = metaDataForTheme(theme);
+    KPluginMetaData data = metaDataForTheme(theme);
+    if (!data.isValid()) {
+        data = metaDataForTheme(QStringLiteral("default"));
         if (!data.isValid()) {
-            data = metaDataForTheme(QStringLiteral("default"));
-            if (!data.isValid()) {
-                return;
-            }
-
-            theme = QLatin1String(ThemePrivate::defaultTheme);
+            return;
         }
+
+        theme = QLatin1String(ThemePrivate::defaultTheme);
     }
 
     // check again as ThemePrivate::defaultTheme might be empty
@@ -490,10 +433,8 @@ void ThemePrivate::setThemeName(const QString &tempThemeName, bool writeSettings
     themeName = theme;
 
     // load the color scheme config
-    const QString colorsFile = realTheme
-        ? QStandardPaths::locate(QStandardPaths::GenericDataLocation,
-                                 QLatin1String(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1String("/colors"))
-        : QString();
+    const QString colorsFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
+                                                      QLatin1String(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1String("/colors"));
 
     // qCDebug(LOG_PLASMA) << "we're going for..." << colorsFile << "*******************";
 
@@ -515,58 +456,56 @@ void ThemePrivate::setThemeName(const QString &tempThemeName, bool writeSettings
     hasWallpapers = !QStandardPaths::locate(QStandardPaths::GenericDataLocation, wallpaperPath, QStandardPaths::LocateDirectory).isEmpty();
 
     // load the wallpaper settings, if any
-    if (realTheme) {
-        pluginMetaData = metaDataForTheme(theme);
-        KSharedConfigPtr metadata = configForTheme(theme);
+    pluginMetaData = metaDataForTheme(theme);
+    KSharedConfigPtr metadata = configForTheme(theme);
 
-        processContrastSettings(metadata);
-        processBlurBehindSettings(metadata);
-        processAdaptiveTransparencySettings(metadata);
+    processContrastSettings(metadata);
+    processBlurBehindSettings(metadata);
+    processAdaptiveTransparencySettings(metadata);
 
-        processWallpaperSettings(metadata);
+    processWallpaperSettings(metadata);
 
+    KConfigGroup cg(metadata, QStringLiteral("Settings"));
+    QString fallback = cg.readEntry("FallbackTheme", QString());
+
+    fallbackThemes.clear();
+    while (!fallback.isEmpty() && !fallbackThemes.contains(fallback)) {
+        fallbackThemes.append(fallback);
+
+        KSharedConfigPtr metadata = configForTheme(fallback);
         KConfigGroup cg(metadata, QStringLiteral("Settings"));
-        QString fallback = cg.readEntry("FallbackTheme", QString());
+        fallback = cg.readEntry("FallbackTheme", QString());
+    }
 
-        fallbackThemes.clear();
-        while (!fallback.isEmpty() && !fallbackThemes.contains(fallback)) {
-            fallbackThemes.append(fallback);
+    if (!fallbackThemes.contains(QLatin1String(ThemePrivate::defaultTheme))) {
+        fallbackThemes.append(QLatin1String(ThemePrivate::defaultTheme));
+    }
 
-            KSharedConfigPtr metadata = configForTheme(fallback);
-            KConfigGroup cg(metadata, QStringLiteral("Settings"));
-            fallback = cg.readEntry("FallbackTheme", QString());
+    for (const QString &theme : std::as_const(fallbackThemes)) {
+        KSharedConfigPtr metadata = configForTheme(theme);
+        processWallpaperSettings(metadata);
+    }
+
+    // Check for what Plasma version the theme has been done
+    // There are some behavioral differences between KDE4 Plasma and Plasma 5
+    const QString apiVersion = pluginMetaData.value(u"X-Plasma-API");
+    apiMajor = 1;
+    apiMinor = 0;
+    apiRevision = 0;
+    if (!apiVersion.isEmpty()) {
+        const QList<QStringView> parts = QStringView(apiVersion).split(QLatin1Char('.'));
+        if (!parts.isEmpty()) {
+            apiMajor = parts.value(0).toInt();
         }
-
-        if (!fallbackThemes.contains(QLatin1String(ThemePrivate::defaultTheme))) {
-            fallbackThemes.append(QLatin1String(ThemePrivate::defaultTheme));
+        if (parts.count() > 1) {
+            apiMinor = parts.value(1).toInt();
         }
-
-        for (const QString &theme : std::as_const(fallbackThemes)) {
-            KSharedConfigPtr metadata = configForTheme(theme);
-            processWallpaperSettings(metadata);
-        }
-
-        // Check for what Plasma version the theme has been done
-        // There are some behavioral differences between KDE4 Plasma and Plasma 5
-        const QString apiVersion = pluginMetaData.value(u"X-Plasma-API");
-        apiMajor = 1;
-        apiMinor = 0;
-        apiRevision = 0;
-        if (!apiVersion.isEmpty()) {
-            const QList<QStringView> parts = QStringView(apiVersion).split(QLatin1Char('.'));
-            if (!parts.isEmpty()) {
-                apiMajor = parts.value(0).toInt();
-            }
-            if (parts.count() > 1) {
-                apiMinor = parts.value(1).toInt();
-            }
-            if (parts.count() > 2) {
-                apiRevision = parts.value(2).toInt();
-            }
+        if (parts.count() > 2) {
+            apiRevision = parts.value(2).toInt();
         }
     }
 
-    if (realTheme && isDefault && writeSettings) {
+    if (isDefault && writeSettings) {
         // we're the default theme, let's save our status
         KConfigGroup &cg = config();
         cg.writeEntry("name", themeName);
@@ -574,7 +513,7 @@ void ThemePrivate::setThemeName(const QString &tempThemeName, bool writeSettings
     }
 
     if (emitChanged) {
-        scheduleThemeChangeNotification(PixmapCache | SvgElementsCache);
+        scheduleThemeChangeNotification();
     }
 }
 
